<?php

/**
 * This class handles the metabox for this plugin
 *
 * @link       https://crowdaa.com
 * @since      1.0.0
 *
 * @package    Crowdaa_Sync_Meta_Box
 * @subpackage Crowdaa_Sync_Meta_Box/admin
 * @author     Crowdaa <contact@crowdaa.com>
 */
class Crowdaa_Sync_Meta_Box {
  /**
   * Initialize the class and set its properties.
   *
   * @since    1.0.0
   */
  public function __construct() {
    add_action('add_meta_boxes', [$this, 'add_meta_boxes'], 10, 2);
    add_action('save_post', [$this, 'save_meta_boxes']);
  }


  /**
   * Add a meta box
   *
   * @since    1.0.0
   * @return boolean
   */
  function add_meta_boxes() {
    add_meta_box(
      'crowdaa_feat_img_slider',
      'Crowdaa-sync: Featured Image Gallery',
      [$this, 'print_feat_img_slider_box'],
      'post',
      'advanced',
      'high',
      array(
        '__back_compat_meta_box' => false,
      )
    );
    add_meta_box(
      'crowdaa-sync-notification-send',
      'Crowdaa-sync: Send notifications when published?',
      [$this, 'print_send_notification_checkbox'],
      'post',
      'advanced',
      'high',
      array(
        '__back_compat_meta_box' => false,
      )
    );
    add_meta_box(
      'crowdaa-sync-display-options',
      'Crowdaa-sync: Mobile display options',
      [$this, 'print_display_options_checkbox'],
      'post',
      'advanced',
      'high',
      array(
        '__back_compat_meta_box' => false,
      )
    );
  }

  /**
   * Gallery form
   *
   * @since    1.0.0
   * @param $name
   * @param $value
   * @return boolean
   */
  function render_image_uploader_field($name, $value = '') {
    ?>
    <p>
      <i>
        <?php
          esc_html_e('Set Images or Videos for the Gallery. Hold Ctrl to select multiple items. Shortcode - [display_gallery]', CROWDAA_SYNC_PLUGIN_NAME);
        ?>
      </i>
    </p>

    <label>
      <div class="gallery-screenshot clearfix">
        <?php
        if ($value) {
          $ids = explode(',', $value);
          foreach ($ids as $attachment_id) {
            $img = wp_get_attachment_image_src($attachment_id, 'thumbnail');
            if ($img) {
              ?>
                <div class="screen-thumb">
                  <img src="<?php echo esc_url($img[0]) ?>" />
                </div>
              <?php
            } else {
              $video_url = wp_get_attachment_url($attachment_id);
              $video_title = get_the_title($attachment_id);
              ?>
                <div
                  class="attachment-preview video js--select-attachment type-video subtype-mp4 landscape"
                  res="<?php echo esc_attr($video_url) ?>"
                >
                  <div class="thumbnail">
                    <div class="centered">
                      <img src="../wp-includes/images/media/video.png" class="icon" draggable="false" alt="" />
                    </div>
                    <div class="filename">
                      <div>
                        <?php echo esc_html($video_title) ?>
                      </div>
                    </div>
                  </div>
                </div>
              <?php
            }
          }
        }
        ?>
      </div>

      <input
        id="edit-gallery"
        class="button upload_gallery_button"
        type="button"
        value="<?php esc_html_e('Edit Gallery', 'crowdaa'); ?>"
      />
      <input
        id="clear-gallery"
        class="button upload_gallery_button"
        type="button"
        value="<?php esc_html_e('Clear', 'crowdaa'); ?>"
      />
      <input
        type="hidden"
        name="<?php echo esc_attr($name); ?>"
        id="<?php echo esc_attr($name); ?>"
        value="<?php echo esc_attr($value); ?>"
        class="gallery_values"
      />
    </label>
    <?php
  }

  /**
   * Image selector meta box
   *
   * @since    1.0.0
   * @param $post
   * @return string
   */
  function print_feat_img_slider_box($post) {
    wp_nonce_field('save_feat_gallery', 'feat_gallery_nonce');

    $meta_key = 'second_featured_img';
    $this->render_image_uploader_field($meta_key, get_post_meta($post->ID, $meta_key, true));
  }

  /**
   * Notification checkbox meta box
   *
   * @since    1.0.0
   * @param $post
   * @return string
   */
  function print_send_notification_checkbox($post) {
    $notification_content = get_post_meta($post->ID, 'crowdaa_notification_content', true) ?: '';
    $notification_title = get_post_meta($post->ID, 'crowdaa_notification_title', true) ?: '';
    wp_nonce_field('crowdaa_notification_checkbox', 'crowdaa_notification_checkbox_nonce');

    ?>
    <p>
      <i>
        <?php
          esc_html_e('Check this box if you want to send a notification when the article is published or right now if it is already published.', CROWDAA_SYNC_PLUGIN_NAME);
        ?>
      </i>
    </p>

    <div class="crowdaa-meta-box-input-row">
      <label>
        <?php
          esc_html_e('Send notification :', CROWDAA_SYNC_PLUGIN_NAME);
        ?>
        <input
          type="checkbox"
          name="crowdaa_send_post_notification"
        />
      </label>
    </div>
    <div class="crowdaa-meta-box-input-row">
      <label>
        <?php
          esc_html_e('Notification text (When both are not set, defaults to the article title & first characters) :', CROWDAA_SYNC_PLUGIN_NAME);
        ?>
        <br />
        <input
          style="display: inline-block; width: 500px;"
          type="text"
          name="crowdaa_send_post_notification_title"
          placeholder="<?php esc_attr_e('Notification title', CROWDAA_SYNC_PLUGIN_NAME); ?>"
          value="<?php echo esc_attr($notification_title); ?>"
        />
        <br />
        <input
          style="display: inline-block; width: 500px;"
          type="text"
          name="crowdaa_send_post_notification_content"
          placeholder="<?php esc_attr_e('Notification content', CROWDAA_SYNC_PLUGIN_NAME); ?>"
          value="<?php echo esc_attr($notification_content); ?>"
        />
      </label>
    </div>
    <?php
  }

  function print_display_options_checkbox($post) {
    $display_option_fullscreen = (get_post_meta($post->ID, 'display_option_fullscreen', true) === 'yes');
    wp_nonce_field('crowdaa_display_options_checkbox', 'crowdaa_display_options_checkboxes_nonce');

    ?>
    <p>
      <i>
        <?php
          esc_html_e('Display options for the mobile view of this article.', CROWDAA_SYNC_PLUGIN_NAME);
        ?>
      </i>
    </p>

    <div class="crowdaa-meta-box-input-row">
      <label>
        <?php
          esc_html_e('Display the article as full screen :', CROWDAA_SYNC_PLUGIN_NAME);
        ?>
        <input
          type="checkbox"
          name="crowdaa_display_option_fullscreen"
          <?php if ($display_option_fullscreen) echo esc_attr('checked'); ?>
        />
      </label>
    </div>
    <?php
  }

  function save_gallery_meta_box($post_id) {
    if (!isset($_POST['feat_gallery_nonce'])) {
      return;
    } else if (!wp_verify_nonce($_POST['feat_gallery_nonce'], 'save_feat_gallery')) {
      return;
    } else if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
      return;
    } else if (!current_user_can('edit_post', $post_id)) {
      return;
    }

    if (isset($_POST['second_featured_img'])) {
      $imgs = array_reduce(explode(',', $_POST['second_featured_img']), function($imgs, $id) {
        $intVal = (int) $id;
        if ($intVal > 0) {
          $imgs[] = $intVal;
        }
        return ($imgs);
      }, []);

      update_post_meta($post_id, 'second_featured_img', implode(',', $imgs));
    } else {
      update_post_meta($post_id, 'second_featured_img', '');
    }
  }

  function save_notifications_meta_box($post_id) {
    if (!isset($_POST['crowdaa_notification_checkbox_nonce'])) {
      return;
    } else if (!wp_verify_nonce($_POST['crowdaa_notification_checkbox_nonce'], 'crowdaa_notification_checkbox')) {
      return;
    } else if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
      return;
    } else if (!current_user_can('edit_post', $post_id)) {
      return;
    }

    if (isset($_POST['crowdaa_send_post_notification'])) {
      update_post_meta($post_id, 'crowdaa_notification_send', 'yes');
      update_post_meta($post_id, 'crowdaa_notification_sent', 'no');
      update_post_meta($post_id, 'crowdaa_notification_title', $_POST['crowdaa_send_post_notification_title'] ?: '');
      update_post_meta($post_id, 'crowdaa_notification_content', $_POST['crowdaa_send_post_notification_content'] ?: '');
    }
  }

  function save_display_options_meta_box($post_id) {
    if (!isset($_POST['crowdaa_display_options_checkboxes_nonce'])) {
      return;
    } else if (!wp_verify_nonce($_POST['crowdaa_display_options_checkboxes_nonce'], 'crowdaa_display_options_checkbox')) {
      return;
    } else if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
      return;
    } else if (!current_user_can('edit_post', $post_id)) {
      return;
    }

    if (isset($_POST['crowdaa_display_option_fullscreen'])) {
      update_post_meta($post_id, 'display_option_fullscreen', 'yes');
    } else {
      update_post_meta($post_id, 'display_option_fullscreen', 'no');
    }
  }

  /**
   * Save meta boxes data
   *
   * @since    1.0.0
   * @param $post_id
   * @return boolean
   */
  function save_meta_boxes($post_id) {
    $this->save_gallery_meta_box($post_id);
    $this->save_notifications_meta_box($post_id);
    $this->save_display_options_meta_box($post_id);
  }
}
